// BaseUvc.cpp
#include "USBHostConf.h"
#include "USBHost.h"
#include "USBIsochronous.h"
#include "BaseUvc.h"

void BaseUvc::poll(int millisec)
{
    HCITD* itd = m_isoEp->isochronousReceive(millisec);
    if (itd) {
        uint8_t cc = itd->ConditionCode();
        report_cc_count[cc]++;
        if (cc == 0) {
            uint16_t frame = itd->StartingFrame();
            int fc = itd->FrameCount();
            uint8_t* buf = const_cast<uint8_t*>(itd->buf); 
            int mps = m_isoEp->m_PacketSize;
            for(int i = 0; i < fc; i++) {
                uint16_t psw = itd->OffsetPSW[i];
                cc = psw>>12;
                if (cc == 0 || cc == 9) {
                    int len = psw & 0x7ff;
                    onResult(frame, buf, len);
               }
               report_ps_cc_count[cc]++;
               buf += mps;
               frame++;
            }
        }
        delete itd;
    }
}

USB_TYPE BaseUvc::Control(int req, int cs, int index, uint8_t* buf, int size)
{
    if (req == SET_CUR) {    
        return host->controlWrite(dev,
                    USB_HOST_TO_DEVICE | USB_REQUEST_TYPE_CLASS | USB_RECIPIENT_INTERFACE, 
                    req, cs<<8, index, buf, size);
    }
    return host->controlRead(dev,
                USB_DEVICE_TO_HOST | USB_REQUEST_TYPE_CLASS | USB_RECIPIENT_INTERFACE, 
                req, cs<<8, index, buf, size);
}

USB_TYPE BaseUvc::setInterfaceAlternate(uint8_t intf, uint8_t alt)
{
    return host->controlWrite(dev, USB_HOST_TO_DEVICE | USB_RECIPIENT_INTERFACE,
                                   SET_INTERFACE, alt, intf, NULL, 0);
}

void BaseUvc::onResult(uint16_t frame, uint8_t* buf, int len)
{
  if(m_pCbItem && m_pCbMeth)
    (m_pCbItem->*m_pCbMeth)(frame, buf, len);
  else if(m_pCb)
    m_pCb(frame, buf, len);
}

void BaseUvc::setOnResult( void (*pMethod)(uint16_t, uint8_t*, int) )
{
    m_pCb = pMethod;
    m_pCbItem = NULL;
    m_pCbMeth = NULL;
}
    
void BaseUvc::clearOnResult()
{
    m_pCb = NULL;
    m_pCbItem = NULL;
    m_pCbMeth = NULL;
}


